/*
 * Copyright (c) 2016-2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gio/gio.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define LOG_TAG "LIBCRASH-SERVICE"

#include "shared/log.h"

#define BUS_NAME "org.tizen.system.crash.livedump"
#define OBJECT_PATH "/Org/Tizen/System/Crash/Livedump"
#define INTERFACE_NAME "org.tizen.system.crash.livedump"
#define METHOD_NAME "livedump_pid"

static GDBusConnection* dbus_init(void)
{
	GError *error = NULL;
	GDBusConnection *conn = g_bus_get_sync(G_BUS_TYPE_SYSTEM, NULL, &error);

	if (!conn || error) {
		_E("Failed to get dbus: %s", error ? error->message : "");
		if (error)
			g_error_free(error);
		return NULL;
	}

	return conn;
}

bool livedump_pid(pid_t pid, const char *dump_reason, char *report_path, size_t report_path_len)
{
	bool res = true;
	GDBusConnection *conn = dbus_init();

	if (!conn)
		return false;

	GVariant *parameters = g_variant_new("(is)", pid, dump_reason);

	GError *error = NULL;
	GVariant *reply = g_dbus_connection_call_sync(conn,
	                               BUS_NAME,
	                               OBJECT_PATH,
	                               INTERFACE_NAME,
	                               METHOD_NAME,
	                               parameters,
	                               G_VARIANT_TYPE("(s)"),
	                               G_DBUS_CALL_FLAGS_NONE,
	                               -1,
	                               NULL,
	                               &error);
	if (!reply || error) {
		_E("Error while calling livedump_pid via dbus (pid=%d, reason=%s): %s", pid, dump_reason, error ? error->message : "");
		if (error)
			g_error_free(error);
		res = false;
		goto exit;
	}

	if (!g_variant_is_of_type(reply, G_VARIANT_TYPE("(s)"))) {
		_E("reply is not of the correct type (s)");
		res = false;
		goto exit;
	}

	gchar *reply_str;
	g_variant_get(reply, "(&s)", &reply_str);
	if (strlen(reply_str) <= (report_path_len + 1)) {
		strncpy(report_path, reply_str, report_path_len);
	} else {
		_E("Report path (%s) is longer than report_path_len", reply_str);
		res = false;
	}
exit:
	g_object_unref(conn);
	return res;
}
